//
// Copyright (c) 2010 smdn <smdn@smdn.jp>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;

namespace Smdn.Net.Pop3.Client {
  partial class PopClient {
    [Obsolete(ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task<IReadOnlyList<PopMessage>> GetMessagesAsync(CancellationToken cancellationToken)
      => GetMessagesAsync(getUniqueId: defaultGetUniqueId, cancellationToken: cancellationToken);

    public Task<IReadOnlyList<PopMessage>> GetMessagesAsync(bool getUniqueId = defaultGetUniqueId,
                                                            CancellationToken cancellationToken = default)
    {
      return StartTaskExclusively(() => GetMessagesCore(getUniqueId),
                                  cancellationToken);
    }

    [Obsolete(ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task<PopMessage> GetMessageAsync(long messageNumber,
                                            CancellationToken cancellationToken)
      => GetMessageAsync(messageNumber, getUniqueId: defaultGetUniqueId, cancellationToken: cancellationToken);

    public Task<PopMessage> GetMessageAsync(long messageNumber,
                                            bool getUniqueId = defaultGetUniqueId,
                                            CancellationToken cancellationToken = default)
    {
      if (messageNumber <= 0L)
        throw ExceptionUtils.CreateArgumentMustBeNonZeroPositive(nameof(messageNumber), messageNumber);

      return StartTaskExclusively(() => GetMessageByMessageNumber(messageNumber, getUniqueId),
                                  cancellationToken);
    }

    [Obsolete(ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task<PopMessage> GetFirstMessageAsync(CancellationToken cancellationToken)
      => GetFirstMessageAsync(getUniqueId: defaultGetUniqueId, cancellationToken: cancellationToken);

    public Task<PopMessage> GetFirstMessageAsync(bool getUniqueId = defaultGetUniqueId,
                                                 CancellationToken cancellationToken = default)
    {
      return GetMessageAsync(1L,
                             getUniqueId,
                             cancellationToken);
    }

    [Obsolete(ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task<PopMessage> GetLastMessageAsync(CancellationToken cancellationToken)
      => GetLastMessageAsync(getUniqueId: defaultGetUniqueId, cancellationToken: cancellationToken);

    public Task<PopMessage> GetLastMessageAsync(bool getUniqueId = defaultGetUniqueId,
                                                CancellationToken cancellationToken = default)
    {
      return StartTaskExclusively(() => GetLastMessageCore(getUniqueId),
                                  cancellationToken);
    }

    public Task<PopMessage> GetMessageAsync(string uniqueId,
                                            CancellationToken cancellationToken = default)
    {
      uniqueId = GetValidatedUniqueId(uniqueId);

      return StartTaskExclusively(() => GetMessageByUniqueId(uniqueId),
                                  cancellationToken);
    }

    public Task UpdateStatusAsync()
    {
      return StartTaskExclusively(UpdateStatusCore,
                                  CancellationToken.None);
    }

    public Task CancelDeleteAsync()
    {
      return StartTaskExclusively(CancelDeleteCore,
                                  CancellationToken.None);
    }

    public Task KeepAliveAsync()
    {
      return StartTaskExclusively(KeepAliveCore,
                                  CancellationToken.None);
    }
  }
}
