using System;

using Smdn.Net.MessageAccessProtocols.Strings;
using Smdn.Net.Pop3.Protocol.Transmission;
using Smdn.Net.Pop3.Protocol.Transmission.Transaction;

namespace Smdn.Net.Pop3.Protocol.Session {
  partial class PopSession {
    /*
     * transaction methods : any state
     */

    /// <summary>sends QUIT command</summary>
    /// <remarks>valid in any state</remarks>
    public PopCommandResult Quit()
    {
      RejectNonConnectedState();

      if (State == PopSessionState.Transaction)
        TransitStateTo(PopSessionState.Update);

      using (var t = new QuitTransaction(connection)) {
        try {
          return ProcessTransaction(t);
        }
        finally {
          /*
           * 6. The UPDATE State
           *    QUIT
           *       Whether the removal was successful or not, the server
           *       then releases any exclusive-access lock on the maildrop
           *       and closes the TCP connection.
           */
          CloseConnection();
        }
      }
    }

    /// <summary>sends CAPA command</summary>
    /// <remarks>valid in any state</remarks>
    public PopCommandResult Capa()
    {
      return Capa(out _);
    }

    /// <summary>sends CAPA command</summary>
    /// <remarks>valid in any state</remarks>
    public PopCommandResult Capa(out IReadOnlyStringSet<PopCapability> capabilities)
    {
      RejectNonConnectedState();

      capabilities = null;

      using (var t = new CapaTransaction(connection)) {
        if (ProcessTransaction(t).IsSuccess) {
          SetServerCapabilities(t.TransactionResult);
          capabilities = t.TransactionResult;
        }

        return t.Result;
      }
    }
  }
}
