// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.MessageAccessProtocols.Strings;

namespace Smdn.Net.Pop3.Protocol.Strings {
  // enum types:
  // * PopStringEnum
  //     => handles string constants
  //     PopAuthenticationMechanism
  //       => handles authentication mechanisms
  //     PopCapability
  //       => handles capability constants

#if SERIALIZATION
  [Serializable]
#endif
  public abstract class PopEnumValue : IEquatable<string> {
    public virtual string Value { get; }

    private protected PopEnumValue(string @value)
    {
      if (@value == null)
        throw new ArgumentNullException(nameof(value));
      if (@value.Length == 0)
        throw ExceptionUtils.CreateArgumentMustBeNonEmptyString(nameof(value));

      Value = @value;
    }

#region "defined constants"
    private protected static PopEnumValueSet<TEnumValue> CreateDefinedConstantsSet<TEnumValue>() where TEnumValue : PopEnumValue
    {
      return new PopEnumValueSet<TEnumValue>(TypedStringUtils.GetDefinedConstants<TEnumValue>(), asReadOnly: true);
    }
#endregion

#region "equatable"
    private protected static bool Equals<TEnumValue>(TEnumValue x, TEnumValue y) where TEnumValue : PopEnumValue
      => PopEnumValueEqualityComparer<TEnumValue>.Default.Equals(x, y);

    public bool Equals(string other) => PopEnumValueEqualityComparer<PopEnumValue>.Default.Equals(this, other);
    public override int GetHashCode() => PopEnumValueEqualityComparer<PopEnumValue>.Default.GetHashCode(this);

    public override bool Equals(object obj)
    {
      if (obj == null)
        return false;
      if (obj is string str)
        return Equals(str);
      if (obj is PopEnumValue enumValue)
        return GetType() == enumValue.GetType() && PopEnumValueEqualityComparer<PopEnumValue>.Default.Equals(this, enumValue);

      return false;
    }
#endregion

    public override string ToString() => Value;
  }
}
