// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Trace = Smdn.Net.Pop3.Client.Trace;

namespace Smdn.Net.Pop3.Protocol.Transmission.Transaction {
  internal sealed class GreetingTransaction : PopTransactionBase<string> {
    public GreetingTransaction(PopConnection connection)
      : base(connection)
    {
    }

    protected override PopCommand PrepareCommand()
    {
      Trace.Log(this, "waiting for server greeting response");

      // send nothing
      return null;
    }

    private static PopConnectionException CreateException(string message)
      => new PopConnectionException($"session establishment failed ({message})");

    protected override void OnFollowingResponseReceived(PopFollowingResponse following)
      => throw CreateException("unexpected following line response");

    protected override void OnTerminationResponse(PopTerminationResponse termination)
      => throw CreateException("unexpected termination response");

    protected override void OnContinuationRequestReceived(PopContinuationRequest continuationRequest)
      => throw CreateException("unexpected continuation request");

    protected override void OnStatusResponseReceived(PopStatusResponse status)
    {
      if (status.Status == PopStatusIndicator.Positive) {
        var timestamp = PopResponseParser.FromGreetingBanner(status);

        if (!string.IsNullOrEmpty(timestamp)) {
          Finish(status, timestamp);
          return;
        }
      }

      base.OnStatusResponseReceived(status);
    }
  }
}
