using System;
using System.IO;
using NUnit.Framework;

using Smdn.IO.Streams.LineOriented;
using Smdn.Net.Pop3.Protocol.Transmission;
using Smdn.Text.Encodings;

namespace Smdn.Net.Pop3.Protocol.DataStructures {
  [TestFixture]
  public class PopUniqueIdListingTests {
    private static PopResponse GetResponse(string resp)
    {
      var receiver = new PopPseudoResponseReceiver();

      receiver.SetResponse(resp);

      return receiver.ReceiveResponse();
    }

    [Test]
    public void TestEquatableLong()
    {
      var response = GetResponse("+OK 1 xxx\r\n") as PopStatusResponse;
      var uidl = PopResponseParser.FromUidl(response);

      Assert.IsTrue (uidl.Equals(1L));
      Assert.IsFalse(uidl.Equals(0L));
    }

    [Test]
    public void TestEquatableString()
    {
      var response = GetResponse("+OK 1 xxx\r\n") as PopStatusResponse;
      var uidl = PopResponseParser.FromUidl(response);

      Assert.IsTrue (uidl.Equals("xxx"));
      Assert.IsFalse(uidl.Equals("xxX"));
      Assert.IsFalse(uidl.Equals("XXX"));
      Assert.IsFalse(uidl.Equals("unique-id"));
    }
  }
}
