using System;
using System.IO;
using NUnit.Framework;

using Smdn.IO.Streams.LineOriented;
using Smdn.Text.Encodings;

namespace Smdn.Net.Pop3.Protocol.Transmission {
  [TestFixture]
  public class PopCommandSenderTests {
    [SetUp]
    public void Setup()
    {
      baseStream = new MemoryStream();
      sender = new PopCommandSender(new LineOrientedBufferedStream(baseStream));
    }

    [TearDown]
    public void TearDown()
    {
      // nothing to do
    }

    private string SendCommand(PopCommand command)
    {
      baseStream.Seek(0, SeekOrigin.Begin);

      sender.Send(command);

      baseStream.Seek(0, SeekOrigin.Begin);

      var reader = new StreamReader(baseStream, OctetEncoding.EightBits);

      return reader.ReadToEnd();
    }

    [Test]
    public void TestSendCommand()
    {
      Assert.AreEqual("RSET\r\n",
                      SendCommand(PopCommand.Create("RSET")));
    }

    [Test]
    public void TestSendCommandWithArguments()
    {
      Assert.AreEqual("TOP 1 10\r\n",
                      SendCommand(PopCommand.Create("TOP", new[] {"1", "10"})));
    }

    [Test]
    public void TestSendContinuation()
    {
      Assert.AreEqual("dGVzdAB0ZXN0AHRlc3Q=\r\n",
                      SendCommand(PopCommand.CreateContinuation("dGVzdAB0ZXN0AHRlc3Q=")));
    }

    [Test]
    public void TestSendCancelResponse()
    {
      Assert.AreEqual("*\r\n",
                      SendCommand(PopCommand.CreateCancelResponse()));
    }

    private MemoryStream baseStream;
    private PopCommandSender sender;
  }
}
