using System;
using NUnit.Framework;

using Smdn.Text;

namespace Smdn.Net.Pop3.Protocol.Transmission {
  [TestFixture]
  public class PopTextParserTests {
    [Test]
    public void TestToCapability()
    {
      Assert.AreEqual(PopCapability.Top, PopTextParser.ToCapability(new[] {
        ByteString.CreateImmutable("TOP"),
      }));
      Assert.AreEqual(new PopCapability("LOGIN-DELAY", "900"), PopTextParser.ToCapability(new[] {
        ByteString.CreateImmutable("LOGIN-DELAY"),
        ByteString.CreateImmutable("900")
      }));
    }

    [Test]
    public void TestToCapability_TagEmpty()
    {
      Assert.Throws<PopMalformedTextException>(() => {
        PopTextParser.ToCapability(new[] {
          ByteString.CreateImmutable(string.Empty),
          ByteString.CreateImmutable("ARG")
        });
      });
    }

    [Test]
    public void TestToCapability_OneOfArgumentsEmpty()
    {
      Assert.Throws<PopMalformedTextException>(() => {
        PopTextParser.ToCapability(new[] {
          ByteString.CreateImmutable("LOGIN-DELAY"),
          ByteString.CreateImmutable(string.Empty)
        });
      });

      Assert.Throws<PopMalformedTextException>(() => {
        PopTextParser.ToCapability(new[] {
          ByteString.CreateImmutable("LOGIN-DELAY"),
          ByteString.CreateImmutable("900"),
          ByteString.CreateImmutable(string.Empty)
        });
      });
    }

    [Test]
    public void TestToDropListing()
    {
      var dropListing = PopTextParser.ToDropListing(
        ByteString.CreateImmutable("2"),
        ByteString.CreateImmutable("320")
        //ByteString.CreateImmutable("undocumented-extension-value"),
      );

      Assert.AreEqual(2L, dropListing.MessageCount);
      Assert.AreEqual(320L, dropListing.SizeInOctets);
    }

    [Test]
    public void TestToScanListing()
    {
      var scanListing = PopTextParser.ToScanListing(
        ByteString.CreateImmutable("1"),
        ByteString.CreateImmutable("120")
        //ByteString.CreateImmutable("undocumented-extension-value"),
      );

      Assert.AreEqual(1L, scanListing.MessageNumber);
      Assert.AreEqual(120L, scanListing.SizeInOctets);
    }

    [Test]
    public void TestToUniqueIdListing()
    {
      var uniqueIdListing = PopTextParser.ToUniqueIdListing(
        ByteString.CreateImmutable("1"),
        ByteString.CreateImmutable("whqtswO00WBw418f9t5JxYwZ")
        //ByteString.CreateImmutable("undocumented-extension-value"),
      );

      Assert.AreEqual(1L, uniqueIdListing.MessageNumber);
      Assert.AreEqual("whqtswO00WBw418f9t5JxYwZ", uniqueIdListing.UniqueId);
    }

    [Test]
    public void TestToUniqueIdListing_UniqueIdEmpty()
    {
      Assert.Throws<PopMalformedTextException>(() => {
        PopTextParser.ToUniqueIdListing(
          ByteString.CreateImmutable("1"),
          ByteString.CreateImmutable(string.Empty)
        );
      });
    }

    [Test]
    public void TestToMessageNumber()
    {
      Assert.AreEqual(1L, PopTextParser.ToMessageNumber(ByteString.CreateImmutable("1")));

      Assert.Throws<PopMalformedTextException>(() => PopTextParser.ToMessageNumber(ByteString.CreateImmutable("0")));
    }

    [Test]
    public void TestToNumber()
    {
      Assert.AreEqual(1L, PopTextParser.ToNumber(ByteString.CreateImmutable("1")));
      Assert.AreEqual(1234567890L, PopTextParser.ToNumber(ByteString.CreateImmutable("1234567890")));

      Assert.Throws<PopMalformedTextException>(() => PopTextParser.ToNumber(ByteString.CreateImmutable("123a")));
      Assert.Throws<PopMalformedTextException>(() => PopTextParser.ToNumber(ByteString.CreateImmutable("-123")));
    }
  }
}
